@@ -6,11 +6,12 @@ import (
 	"encoding/json"
 	"fmt"
 	"math/rand"
-	"reflect"
 	"strconv"
 	"strings"
 	"testing"
 	"time"
+
+	"github.com/tidwall/pretty"
 )
 
 // TestRandomData is a fuzzing test that throws random data at the Parse
@@ -55,14 +56,16 @@ func TestRandomValidStrings(t *testing.T) {
 		}
 		token := Get(`{"str":`+string(sm)+`}`, "str")
 		if token.Type != String || token.Str != su {
-			println("["+token.Raw+"]", "["+token.Str+"]", "["+su+"]", "["+string(sm)+"]")
+			println("["+token.Raw+"]", "["+token.Str+"]", "["+su+"]",
+				"["+string(sm)+"]")
 			t.Fatal("string mismatch")
 		}
 	}
 }
 
 func TestEmoji(t *testing.T) {
-	const input = `{"utf8":"Example emoji, KO: \ud83d\udd13, \ud83c\udfc3 OK: \u2764\ufe0f "}`
+	const input = `{"utf8":"Example emoji, KO: \ud83d\udd13, \ud83c\udfc3 ` +
+		`OK: \u2764\ufe0f "}`
 	value := Get(input, "utf8")
 	var s string
 	json.Unmarshal([]byte(value.Raw), &s)
@@ -142,18 +145,21 @@ var basicJSON = `{"age":100, "name":{"here":"B\\\"R"},
 var basicJSONB = []byte(basicJSON)
 
 func TestTimeResult(t *testing.T) {
-	assert(t, Get(basicJSON, "created").String() == Get(basicJSON, "created").Time().Format(time.RFC3339Nano))
+	assert(t, Get(basicJSON, "created").String() ==
+		Get(basicJSON, "created").Time().Format(time.RFC3339Nano))
 }
 
 func TestParseAny(t *testing.T) {
 	assert(t, Parse("100").Float() == 100)
 	assert(t, Parse("true").Bool())
 	assert(t, Parse("false").Bool() == false)
+	assert(t, Parse("yikes").Exists() == false)
 }
 
 func TestManyVariousPathCounts(t *testing.T) {
 	json := `{"a":"a","b":"b","c":"c"}`
-	counts := []int{3, 4, 7, 8, 9, 15, 16, 17, 31, 32, 33, 63, 64, 65, 127, 128, 129, 255, 256, 257, 511, 512, 513}
+	counts := []int{3, 4, 7, 8, 9, 15, 16, 17, 31, 32, 33, 63, 64, 65, 127,
+		128, 129, 255, 256, 257, 511, 512, 513}
 	paths := []string{"a", "b", "c"}
 	expects := []string{"a", "b", "c"}
 	for _, count := range counts {
@@ -171,7 +177,8 @@ func TestManyVariousPathCounts(t *testing.T) {
 		results := GetMany(json, gpaths...)
 		for i := 0; i < len(paths); i++ {
 			if results[i].String() != expects[i] {
-				t.Fatalf("expected '%v', got '%v'", expects[i], results[i].String())
+				t.Fatalf("expected '%v', got '%v'", expects[i],
+					results[i].String())
 			}
 		}
 	}
@@ -306,10 +313,26 @@ func TestTypes(t *testing.T) {
 	assert(t, (Result{Type: JSON}).Type.String() == "JSON")
 	assert(t, (Result{Type: 100}).Type.String() == "")
 	// bool
-	assert(t, (Result{Type: String, Str: "true"}).Bool())
-	assert(t, (Result{Type: True}).Bool())
+	assert(t, (Result{Type: True}).Bool() == true)
 	assert(t, (Result{Type: False}).Bool() == false)
-	assert(t, (Result{Type: Number, Num: 1}).Bool())
+	assert(t, (Result{Type: Number, Num: 1}).Bool() == true)
+	assert(t, (Result{Type: Number, Num: 0}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "1"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "T"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "t"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "true"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "True"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "TRUE"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "tRuE"}).Bool() == true)
+	assert(t, (Result{Type: String, Str: "0"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "f"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "F"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "false"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "False"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "FALSE"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "fAlSe"}).Bool() == false)
+	assert(t, (Result{Type: String, Str: "random"}).Bool() == false)
+
 	// int
 	assert(t, (Result{Type: String, Str: "1"}).Int() == 1)
 	assert(t, (Result{Type: True}).Int() == 1)
@@ -347,7 +370,8 @@ func TestForEach(t *testing.T) {
 }
 func TestMap(t *testing.T) {
 	assert(t, len(ParseBytes([]byte(`"asdf"`)).Map()) == 0)
-	assert(t, ParseBytes([]byte(`{"asdf":"ghjk"`)).Map()["asdf"].String() == "ghjk")
+	assert(t, ParseBytes([]byte(`{"asdf":"ghjk"`)).Map()["asdf"].String() ==
+		"ghjk")
 	assert(t, len(Result{Type: JSON, Raw: "**invalid**"}.Map()) == 0)
 	assert(t, Result{Type: JSON, Raw: "**invalid**"}.Value() == nil)
 	assert(t, Result{Type: JSON, Raw: "{"}.Map() != nil)
@@ -368,16 +392,21 @@ func TestBasic1(t *testing.T) {
 			value.ForEach(func(key, value Result) bool {
 				switch i {
 				case 0:
-					if key.String() != "firstName" || value.String() != "Brett" {
-						t.Fatalf("expected %v/%v got %v/%v", "firstName", "Brett", key.String(), value.String())
+					if key.String() != "firstName" ||
+						value.String() != "Brett" {
+						t.Fatalf("expected %v/%v got %v/%v", "firstName",
+							"Brett", key.String(), value.String())
 					}
 				case 1:
-					if key.String() != "lastName" || value.String() != "McLaughlin" {
-						t.Fatalf("expected %v/%v got %v/%v", "lastName", "McLaughlin", key.String(), value.String())
+					if key.String() != "lastName" ||
+						value.String() != "McLaughlin" {
+						t.Fatalf("expected %v/%v got %v/%v", "lastName",
+							"McLaughlin", key.String(), value.String())
 					}
 				case 2:
 					if key.String() != "email" || value.String() != "aaaa" {
-						t.Fatalf("expected %v/%v got %v/%v", "email", "aaaa", key.String(), value.String())
+						t.Fatalf("expected %v/%v got %v/%v", "email", "aaaa",
+							key.String(), value.String())
 					}
 				}
 				i++
@@ -395,7 +424,8 @@ func TestBasic2(t *testing.T) {
 	if mtok.String() != "1002.3" {
 		t.Fatalf("expected %v, got %v", "1002.3", mtok.String())
 	}
-	mtok = get(basicJSON, `loggy.programmers.#[firstName != "Brett"].firstName`)
+	mtok = get(basicJSON,
+		`loggy.programmers.#[firstName != "Brett"].firstName`)
 	if mtok.String() != "Jason" {
 		t.Fatalf("expected %v, got %v", "Jason", mtok.String())
 	}
@@ -420,13 +450,16 @@ func TestBasic2(t *testing.T) {
 	}
 	programmers := mtok.Map()["programmers"]
 	if programmers.Array()[1].Map()["firstName"].Str != "Jason" {
-		t.Fatalf("expected %v, got %v", "Jason", mtok.Map()["programmers"].Array()[1].Map()["firstName"].Str)
+		t.Fatalf("expected %v, got %v", "Jason",
+			mtok.Map()["programmers"].Array()[1].Map()["firstName"].Str)
 	}
 }
 func TestBasic3(t *testing.T) {
 	var mtok Result
-	if Parse(basicJSON).Get("loggy.programmers").Get("1").Get("firstName").Str != "Jason" {
-		t.Fatalf("expected %v, got %v", "Jason", Parse(basicJSON).Get("loggy.programmers").Get("1").Get("firstName").Str)
+	if Parse(basicJSON).Get("loggy.programmers").Get("1").
+		Get("firstName").Str != "Jason" {
+		t.Fatalf("expected %v, got %v", "Jason", Parse(basicJSON).
+			Get("loggy.programmers").Get("1").Get("firstName").Str)
 	}
 	var token Result
 	if token = Parse("-102"); token.Num != -102 {
@@ -466,7 +499,8 @@ func TestBasic4(t *testing.T) {
 		t.Fatalf("expected 3, got %v", get(basicJSON, "items.3.tags.#").Num)
 	}
 	if get(basicJSON, "items.3.points.1.#").Num != 2 {
-		t.Fatalf("expected 2, got %v", get(basicJSON, "items.3.points.1.#").Num)
+		t.Fatalf("expected 2, got %v",
+			get(basicJSON, "items.3.points.1.#").Num)
 	}
 	if get(basicJSON, "items.#").Num != 8 {
 		t.Fatalf("expected 6, got %v", get(basicJSON, "items.#").Num)
@@ -522,7 +556,8 @@ func TestBasic5(t *testing.T) {
 	_ = token.Value().(bool)
 	token = get(basicJSON, "noop")
 	if token.String() != `{"what is a wren?":"a bird"}` {
-		t.Fatal("expecting '"+`{"what is a wren?":"a bird"}`+"'", "got", token.String())
+		t.Fatal("expecting '"+`{"what is a wren?":"a bird"}`+"'", "got",
+			token.String())
 	}
 	_ = token.Value().(map[string]interface{})
 
@@ -532,8 +567,10 @@ func TestBasic5(t *testing.T) {
 
 	get(basicJSON, "vals.hello")
 
-	mm := Parse(basicJSON).Value().(map[string]interface{})
-	fn := mm["loggy"].(map[string]interface{})["programmers"].([]interface{})[1].(map[string]interface{})["firstName"].(string)
+	type msi = map[string]interface{}
+	type fi = []interface{}
+	mm := Parse(basicJSON).Value().(msi)
+	fn := mm["loggy"].(msi)["programmers"].(fi)[1].(msi)["firstName"].(string)
 	if fn != "Jason" {
 		t.Fatalf("expecting %v, got %v", "Jason", fn)
 	}
@@ -581,12 +618,18 @@ func TestLess(t *testing.T) {
 	assert(t, Result{Type: Null}.Less(Result{Type: String}, true))
 	assert(t, !Result{Type: False}.Less(Result{Type: Null}, true))
 	assert(t, Result{Type: False}.Less(Result{Type: True}, true))
-	assert(t, Result{Type: String, Str: "abc"}.Less(Result{Type: String, Str: "bcd"}, true))
-	assert(t, Result{Type: String, Str: "ABC"}.Less(Result{Type: String, Str: "abc"}, true))
-	assert(t, !Result{Type: String, Str: "ABC"}.Less(Result{Type: String, Str: "abc"}, false))
-	assert(t, Result{Type: Number, Num: 123}.Less(Result{Type: Number, Num: 456}, true))
-	assert(t, !Result{Type: Number, Num: 456}.Less(Result{Type: Number, Num: 123}, true))
-	assert(t, !Result{Type: Number, Num: 456}.Less(Result{Type: Number, Num: 456}, true))
+	assert(t, Result{Type: String, Str: "abc"}.Less(Result{Type: String,
+		Str: "bcd"}, true))
+	assert(t, Result{Type: String, Str: "ABC"}.Less(Result{Type: String,
+		Str: "abc"}, true))
+	assert(t, !Result{Type: String, Str: "ABC"}.Less(Result{Type: String,
+		Str: "abc"}, false))
+	assert(t, Result{Type: Number, Num: 123}.Less(Result{Type: Number,
+		Num: 456}, true))
+	assert(t, !Result{Type: Number, Num: 456}.Less(Result{Type: Number,
+		Num: 123}, true))
+	assert(t, !Result{Type: Number, Num: 456}.Less(Result{Type: Number,
+		Num: 456}, true))
 	assert(t, stringLessInsensitive("abcde", "BBCDE"))
 	assert(t, stringLessInsensitive("abcde", "bBCDE"))
 	assert(t, stringLessInsensitive("Abcde", "BBCDE"))
@@ -768,7 +811,8 @@ func TestManyBasic(t *testing.T) {
 	testMany(false, "[Point]", "position.type")
 	testMany(false, `[emptya ["world peace"] 31]`, ".a", "loves", "age")
 	testMany(false, `[["world peace"]]`, "loves")
-	testMany(false, `[{"last":"Anderson","first":"Nancy"} Nancy]`, "name", "name.first")
+	testMany(false, `[{"last":"Anderson","first":"Nancy"} Nancy]`, "name",
+		"name.first")
 	testMany(true, `[]`, strings.Repeat("a.", 40)+"hello")
 	res := Get(manyJSON, strings.Repeat("a.", 48)+"a")
 	testMany(true, `[`+res.String()+`]`, strings.Repeat("a.", 48)+"a")
@@ -780,7 +824,8 @@ func testMany(t *testing.T, json string, paths, expected []string) {
 	testManyAny(t, json, paths, expected, true)
 	testManyAny(t, json, paths, expected, false)
 }
-func testManyAny(t *testing.T, json string, paths, expected []string, bytes bool) {
+func testManyAny(t *testing.T, json string, paths, expected []string,
+	bytes bool) {
 	var result []Result
 	for i := 0; i < 2; i++ {
 		var which string
@@ -804,16 +849,21 @@ func testManyAny(t *testing.T, json string, paths, expected []string, bytes bool
 		}
 		for j := 0; j < len(expected); j++ {
 			if result[j].String() != expected[j] {
-				t.Fatalf("Using key '%s' for '%s'\nexpected '%v', got '%v'", paths[j], which, expected[j], result[j].String())
+				t.Fatalf("Using key '%s' for '%s'\nexpected '%v', got '%v'",
+					paths[j], which, expected[j], result[j].String())
 			}
 		}
 	}
 }
 func TestIssue20(t *testing.T) {
-	json := `{ "name": "FirstName", "name1": "FirstName1", "address": "address1", "addressDetails": "address2", }`
+	json := `{ "name": "FirstName", "name1": "FirstName1", ` +
+		`"address": "address1", "addressDetails": "address2", }`
 	paths := []string{"name", "name1", "address", "addressDetails"}
 	expected := []string{"FirstName", "FirstName1", "address1", "address2"}
-	t.Run("SingleMany", func(t *testing.T) { testMany(t, json, paths, expected) })
+	t.Run("SingleMany", func(t *testing.T) {
+		testMany(t, json, paths,
+			expected)
+	})
 }
 
 func TestIssue21(t *testing.T) {
@@ -821,9 +871,14 @@ func TestIssue21(t *testing.T) {
 	           "Level1Field4":4, 
 			   "Level1Field2":{ "Level2Field1":[ "value1", "value2" ], 
 			   "Level2Field2":{ "Level3Field1":[ { "key1":"value1" } ] } } }`
-	paths := []string{"Level1Field1", "Level1Field2.Level2Field1", "Level1Field2.Level2Field2.Level3Field1", "Level1Field4"}
-	expected := []string{"3", `[ "value1", "value2" ]`, `[ { "key1":"value1" } ]`, "4"}
-	t.Run("SingleMany", func(t *testing.T) { testMany(t, json, paths, expected) })
+	paths := []string{"Level1Field1", "Level1Field2.Level2Field1",
+		"Level1Field2.Level2Field2.Level3Field1", "Level1Field4"}
+	expected := []string{"3", `[ "value1", "value2" ]`,
+		`[ { "key1":"value1" } ]`, "4"}
+	t.Run("SingleMany", func(t *testing.T) {
+		testMany(t, json, paths,
+			expected)
+	})
 }
 
 func TestRandomMany(t *testing.T) {
@@ -956,25 +1011,6 @@ var complicatedJSON = `
 }
 `
 
-func TestUnmarshal(t *testing.T) {
-	var s1 ComplicatedType
-	var s2 ComplicatedType
-	if err := json.Unmarshal([]byte(complicatedJSON), &s1); err != nil {
-		t.Fatal(err)
-	}
-	if err := Unmarshal([]byte(complicatedJSON), &s2); err != nil {
-		t.Fatal(err)
-	}
-	if !reflect.DeepEqual(&s1, &s2) {
-		t.Fatal("not equal")
-	}
-	var str string
-	if err := json.Unmarshal([]byte(Get(complicatedJSON, "LeftOut").Raw), &str); err != nil {
-		t.Fatal(err)
-	}
-	assert(t, str == Get(complicatedJSON, "LeftOut").String())
-}
-
 func testvalid(t *testing.T, json string, expect bool) {
 	t.Helper()
 	_, ok := validpayload([]byte(json), 0)
@@ -1038,7 +1074,8 @@ func TestValidBasic(t *testing.T) {
 	testvalid(t, `{"a":"b","a":1}`, true)
 	testvalid(t, `{"a":"b",2"1":2}`, false)
 	testvalid(t, `{"a":"b","a": 1, "c":{"hi":"there"} }`, true)
-	testvalid(t, `{"a":"b","a": 1, "c":{"hi":"there", "easy":["going",{"mixed":"bag"}]} }`, true)
+	testvalid(t, `{"a":"b","a": 1, "c":{"hi":"there", "easy":["going",`+
+		`{"mixed":"bag"}]} }`, true)
 	testvalid(t, `""`, true)
 	testvalid(t, `"`, false)
 	testvalid(t, `"\n"`, true)
@@ -1053,7 +1090,8 @@ func TestValidBasic(t *testing.T) {
 	testvalid(t, string(exampleJSON), true)
 }
 
-var jsonchars = []string{"{", "[", ",", ":", "}", "]", "1", "0", "true", "false", "null", `""`, `"\""`, `"a"`}
+var jsonchars = []string{"{", "[", ",", ":", "}", "]", "1", "0", "true",
+	"false", "null", `""`, `"\""`, `"a"`}
 
 func makeRandomJSONChars(b []byte) {
 	var bb []byte
@@ -1062,6 +1100,7 @@ func makeRandomJSONChars(b []byte) {
 	}
 	copy(b, bb[:len(b)])
 }
+
 func TestValidRandom(t *testing.T) {
 	rand.Seed(time.Now().UnixNano())
 	b := make([]byte, 100000)
@@ -1081,7 +1120,8 @@ func TestValidRandom(t *testing.T) {
 }
 
 func TestGetMany47(t *testing.T) {
-	json := `{"bar": {"id": 99, "mybar": "my mybar" }, "foo": {"myfoo": [605]}}`
+	json := `{"bar": {"id": 99, "mybar": "my mybar" }, "foo": ` +
+		`{"myfoo": [605]}}`
 	paths := []string{"foo.myfoo", "bar.id", "bar.mybar", "bar.mybarx"}
 	expected := []string{"[605]", "99", "my mybar", ""}
 	results := GetMany(json, paths...)
@@ -1090,7 +1130,8 @@ func TestGetMany47(t *testing.T) {
 	}
 	for i, path := range paths {
 		if results[i].String() != expected[i] {
-			t.Fatalf("expected '%v', got '%v' for path '%v'", expected[i], results[i].String(), path)
+			t.Fatalf("expected '%v', got '%v' for path '%v'", expected[i],
+				results[i].String(), path)
 		}
 	}
 }
@@ -1105,7 +1146,8 @@ func TestGetMany48(t *testing.T) {
 	}
 	for i, path := range paths {
 		if results[i].String() != expected[i] {
-			t.Fatalf("expected '%v', got '%v' for path '%v'", expected[i], results[i].String(), path)
+			t.Fatalf("expected '%v', got '%v' for path '%v'", expected[i],
+				results[i].String(), path)
 		}
 	}
 }
@@ -1138,12 +1180,12 @@ func TestNullArray(t *testing.T) {
 	}
 }
 
-func TestRandomGetMany(t *testing.T) {
-	start := time.Now()
-	for time.Since(start) < time.Second*3 {
-		testRandomGetMany(t)
-	}
-}
+// func TestRandomGetMany(t *testing.T) {
+// 	start := time.Now()
+// 	for time.Since(start) < time.Second*3 {
+// 		testRandomGetMany(t)
+// 	}
+// }
 func testRandomGetMany(t *testing.T) {
 	rand.Seed(time.Now().UnixNano())
 	json, keys := randomJSON()
@@ -1187,15 +1229,18 @@ func TestIssue54(t *testing.T) {
 	json := `{"MarketName":null,"Nounce":6115}`
 	r = GetMany(json, "Nounce", "Buys", "Sells", "Fills")
 	if strings.Replace(fmt.Sprintf("%v", r), " ", "", -1) != "[6115]" {
-		t.Fatalf("expected '%v', got '%v'", "[6115]", strings.Replace(fmt.Sprintf("%v", r), " ", "", -1))
+		t.Fatalf("expected '%v', got '%v'", "[6115]",
+			strings.Replace(fmt.Sprintf("%v", r), " ", "", -1))
 	}
 	r = GetMany(json, "Nounce", "Buys", "Sells")
 	if strings.Replace(fmt.Sprintf("%v", r), " ", "", -1) != "[6115]" {
-		t.Fatalf("expected '%v', got '%v'", "[6115]", strings.Replace(fmt.Sprintf("%v", r), " ", "", -1))
+		t.Fatalf("expected '%v', got '%v'", "[6115]",
+			strings.Replace(fmt.Sprintf("%v", r), " ", "", -1))
 	}
 	r = GetMany(json, "Nounce")
 	if strings.Replace(fmt.Sprintf("%v", r), " ", "", -1) != "[6115]" {
-		t.Fatalf("expected '%v', got '%v'", "[6115]", strings.Replace(fmt.Sprintf("%v", r), " ", "", -1))
+		t.Fatalf("expected '%v', got '%v'", "[6115]",
+			strings.Replace(fmt.Sprintf("%v", r), " ", "", -1))
 	}
 }
 
@@ -1226,7 +1271,8 @@ func randomNumber() string {
 	return strconv.FormatInt(int64(rand.Int()%1000000), 10)
 }
 
-func randomObjectOrArray(keys []string, prefix string, array bool, depth int) (string, []string) {
+func randomObjectOrArray(keys []string, prefix string, array bool, depth int) (
+	string, []string) {
 	N := 5 + rand.Int()%5
 	var json string
 	if array {
@@ -1383,7 +1429,8 @@ func TestNumBigString(t *testing.T) {
 	j := fmt.Sprintf(`{"data":[ "hello", "%s" ]}`, i)
 	res := Get(j, "data.1")
 	if res.String() != "900719925474099301239109123101" {
-		t.Fatalf("expected '%v', got '%v'", "900719925474099301239109123101", res.String())
+		t.Fatalf("expected '%v', got '%v'", "900719925474099301239109123101",
+			res.String())
 	}
 }
 
@@ -1422,8 +1469,10 @@ func TestArrayValues(t *testing.T) {
 		output += fmt.Sprintf("%#v", val)
 	}
 	expect := strings.Join([]string{
-		`gjson.Result{Type:3, Raw:"\"PERSON1\"", Str:"PERSON1", Num:0, Index:0}`,
-		`gjson.Result{Type:3, Raw:"\"PERSON2\"", Str:"PERSON2", Num:0, Index:0}`,
+		`gjson.Result{Type:3, Raw:"\"PERSON1\"", Str:"PERSON1", Num:0, ` +
+			`Index:0}`,
+		`gjson.Result{Type:3, Raw:"\"PERSON2\"", Str:"PERSON2", Num:0, ` +
+			`Index:0}`,
 		`gjson.Result{Type:2, Raw:"0", Str:"", Num:0, Index:0}`,
 	}, "\n")
 	if output != expect {
@@ -1431,3 +1480,727 @@ func TestArrayValues(t *testing.T) {
 	}
 
 }
+
+func BenchmarkValid(b *testing.B) {
+	for i := 0; i < b.N; i++ {
+		Valid(complicatedJSON)
+	}
+}
+
+func BenchmarkValidBytes(b *testing.B) {
+	complicatedJSON := []byte(complicatedJSON)
+	for i := 0; i < b.N; i++ {
+		ValidBytes(complicatedJSON)
+	}
+}
+
+func BenchmarkGoStdlibValidBytes(b *testing.B) {
+	complicatedJSON := []byte(complicatedJSON)
+	for i := 0; i < b.N; i++ {
+		json.Valid(complicatedJSON)
+	}
+}
+
+func TestModifier(t *testing.T) {
+	json := `{"other":{"hello":"world"},"arr":[1,2,3,4,5,6]}`
+	opts := *pretty.DefaultOptions
+	opts.SortKeys = true
+	exp := string(pretty.PrettyOptions([]byte(json), &opts))
+	res := Get(json, `@pretty:{"sortKeys":true}`).String()
+	if res != exp {
+		t.Fatalf("expected '%v', got '%v'", exp, res)
+	}
+	res = Get(res, "@pretty|@reverse|@ugly").String()
+	if res != json {
+		t.Fatalf("expected '%v', got '%v'", json, res)
+	}
+	if res := Get(res, "@this").String(); res != json {
+		t.Fatalf("expected '%v', got '%v'", json, res)
+	}
+	if res := Get(res, "other.@this").String(); res != `{"hello":"world"}` {
+		t.Fatalf("expected '%v', got '%v'", json, res)
+	}
+	res = Get(res, "@pretty|@reverse|arr|@reverse|2").String()
+	if res != "4" {
+		t.Fatalf("expected '%v', got '%v'", "4", res)
+	}
+	AddModifier("case", func(json, arg string) string {
+		if arg == "upper" {
+			return strings.ToUpper(json)
+		}
+		if arg == "lower" {
+			return strings.ToLower(json)
+		}
+		return json
+	})
+	res = Get(json, "other|@case:upper").String()
+	if res != `{"HELLO":"WORLD"}` {
+		t.Fatalf("expected '%v', got '%v'", `{"HELLO":"WORLD"}`, res)
+	}
+}
+
+func TestChaining(t *testing.T) {
+	json := `{
+		"info": {
+			"friends": [
+				{"first": "Dale", "last": "Murphy", "age": 44},
+				{"first": "Roger", "last": "Craig", "age": 68},
+				{"first": "Jane", "last": "Murphy", "age": 47}
+			]
+		}
+	  }`
+	res := Get(json, "info.friends|0|first").String()
+	if res != "Dale" {
+		t.Fatalf("expected '%v', got '%v'", "Dale", res)
+	}
+	res = Get(json, "info.friends|@reverse|0|age").String()
+	if res != "47" {
+		t.Fatalf("expected '%v', got '%v'", "47", res)
+	}
+	res = Get(json, "@ugly|i\\nfo|friends.0.first").String()
+	if res != "Dale" {
+		t.Fatalf("expected '%v', got '%v'", "Dale", res)
+	}
+}
+
+func TestSplitPipe(t *testing.T) {
+	split := func(t *testing.T, path, el, er string, eo bool) {
+		t.Helper()
+		left, right, ok := splitPossiblePipe(path)
+		// fmt.Printf("%-40s [%v] [%v] [%v]\n", path, left, right, ok)
+		if left != el || right != er || ok != eo {
+			t.Fatalf("expected '%v/%v/%v', got '%v/%v/%v",
+				el, er, eo, left, right, ok)
+		}
+	}
+
+	split(t, "hello", "", "", false)
+	split(t, "hello.world", "", "", false)
+	split(t, "hello|world", "hello", "world", true)
+	split(t, "hello\\|world", "", "", false)
+	split(t, "hello.#", "", "", false)
+	split(t, `hello.#[a|1="asdf\"|1324"]#\|that`, "", "", false)
+	split(t, `hello.#[a|1="asdf\"|1324"]#|that.more|yikes`,
+		`hello.#[a|1="asdf\"|1324"]#`, "that.more|yikes", true)
+	split(t, `a.#[]#\|b`, "", "", false)
+
+}
+
+func TestArrayEx(t *testing.T) {
+	json := `
+	[
+		{
+			"c":[
+				{"a":10.11}
+			]
+		}, {
+			"c":[
+				{"a":11.11}
+			]
+		}
+	]`
+	res := Get(json, "@ugly|#.c.#[a=10.11]").String()
+	if res != `[{"a":10.11}]` {
+		t.Fatalf("expected '%v', got '%v'", `[{"a":10.11}]`, res)
+	}
+	res = Get(json, "@ugly|#.c.#").String()
+	if res != `[1,1]` {
+		t.Fatalf("expected '%v', got '%v'", `[1,1]`, res)
+	}
+	res = Get(json, "@reverse|0|c|0|a").String()
+	if res != "11.11" {
+		t.Fatalf("expected '%v', got '%v'", "11.11", res)
+	}
+	res = Get(json, "#.c|#").String()
+	if res != "2" {
+		t.Fatalf("expected '%v', got '%v'", "2", res)
+	}
+}
+
+func TestPipeDotMixing(t *testing.T) {
+	json := `{
+		"info": {
+			"friends": [
+				{"first": "Dale", "last": "Murphy", "age": 44},
+				{"first": "Roger", "last": "Craig", "age": 68},
+				{"first": "Jane", "last": "Murphy", "age": 47}
+			]
+		}
+	  }`
+	var res string
+	res = Get(json, `info.friends.#[first="Dale"].last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `info|friends.#[first="Dale"].last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `info|friends.#[first="Dale"]|last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `info|friends|#[first="Dale"]|last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `@ugly|info|friends|#[first="Dale"]|last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `@ugly|info.@ugly|friends|#[first="Dale"]|last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `@ugly.info|@ugly.friends|#[first="Dale"]|last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+}
+
+func TestDeepSelectors(t *testing.T) {
+	json := `{
+		"info": {
+			"friends": [
+				{
+					"first": "Dale", "last": "Murphy",
+					"extra": [10,20,30],
+					"details": {
+						"city": "Tempe",
+						"state": "Arizona"
+					}
+				},
+				{
+					"first": "Roger", "last": "Craig", 
+					"extra": [40,50,60],
+					"details": {
+						"city": "Phoenix",
+						"state": "Arizona"
+					}
+				}
+			]
+		}
+	  }`
+	var res string
+	res = Get(json, `info.friends.#[first="Dale"].extra.0`).String()
+	if res != "10" {
+		t.Fatalf("expected '%v', got '%v'", "10", res)
+	}
+	res = Get(json, `info.friends.#[first="Dale"].extra|0`).String()
+	if res != "10" {
+		t.Fatalf("expected '%v', got '%v'", "10", res)
+	}
+	res = Get(json, `info.friends.#[first="Dale"]|extra|0`).String()
+	if res != "10" {
+		t.Fatalf("expected '%v', got '%v'", "10", res)
+	}
+	res = Get(json, `info.friends.#[details.city="Tempe"].last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+	res = Get(json, `info.friends.#[details.city="Phoenix"].last`).String()
+	if res != "Craig" {
+		t.Fatalf("expected '%v', got '%v'", "Craig", res)
+	}
+	res = Get(json, `info.friends.#[details.state="Arizona"].last`).String()
+	if res != "Murphy" {
+		t.Fatalf("expected '%v', got '%v'", "Murphy", res)
+	}
+}
+
+func TestMultiArrayEx(t *testing.T) {
+	json := `{
+		"info": {
+			"friends": [
+				{
+					"first": "Dale", "last": "Murphy", "kind": "Person",
+					"cust1": true,
+					"extra": [10,20,30],
+					"details": {
+						"city": "Tempe",
+						"state": "Arizona"
+					}
+				},
+				{
+					"first": "Roger", "last": "Craig", "kind": "Person",
+					"cust2": false,
+					"extra": [40,50,60],
+					"details": {
+						"city": "Phoenix",
+						"state": "Arizona"
+					}
+				}
+			]
+		}
+	  }`
+
+	var res string
+
+	res = Get(json, `info.friends.#[kind="Person"]#.kind|0`).String()
+	if res != "Person" {
+		t.Fatalf("expected '%v', got '%v'", "Person", res)
+	}
+	res = Get(json, `info.friends.#.kind|0`).String()
+	if res != "Person" {
+		t.Fatalf("expected '%v', got '%v'", "Person", res)
+	}
+
+	res = Get(json, `info.friends.#[kind="Person"]#.kind`).String()
+	if res != `["Person","Person"]` {
+		t.Fatalf("expected '%v', got '%v'", `["Person","Person"]`, res)
+	}
+	res = Get(json, `info.friends.#.kind`).String()
+	if res != `["Person","Person"]` {
+		t.Fatalf("expected '%v', got '%v'", `["Person","Person"]`, res)
+	}
+
+	res = Get(json, `info.friends.#[kind="Person"]#|kind`).String()
+	if res != `` {
+		t.Fatalf("expected '%v', got '%v'", ``, res)
+	}
+	res = Get(json, `info.friends.#|kind`).String()
+	if res != `` {
+		t.Fatalf("expected '%v', got '%v'", ``, res)
+	}
+
+	res = Get(json, `i*.f*.#[kind="Other"]#`).String()
+	if res != `[]` {
+		t.Fatalf("expected '%v', got '%v'", `[]`, res)
+	}
+}
+
+func TestQueries(t *testing.T) {
+	json := `{
+		"info": {
+			"friends": [
+				{
+					"first": "Dale", "last": "Murphy", "kind": "Person",
+					"cust1": true,
+					"extra": [10,20,30],
+					"details": {
+						"city": "Tempe",
+						"state": "Arizona"
+					}
+				},
+				{
+					"first": "Roger", "last": "Craig", "kind": "Person",
+					"cust2": false,
+					"extra": [40,50,60],
+					"details": {
+						"city": "Phoenix",
+						"state": "Arizona"
+					}
+				}
+			]
+		}
+	  }`
+
+	// numbers
+	assert(t, Get(json, "i*.f*.#[extra.0<11].first").Exists())
+	assert(t, Get(json, "i*.f*.#[extra.0<=11].first").Exists())
+	assert(t, !Get(json, "i*.f*.#[extra.0<10].first").Exists())
+	assert(t, Get(json, "i*.f*.#[extra.0<=10].first").Exists())
+	assert(t, Get(json, "i*.f*.#[extra.0=10].first").Exists())
+	assert(t, !Get(json, "i*.f*.#[extra.0=11].first").Exists())
+	assert(t, Get(json, "i*.f*.#[extra.0!=10].first").String() == "Roger")
+	assert(t, Get(json, "i*.f*.#[extra.0>10].first").String() == "Roger")
+	assert(t, Get(json, "i*.f*.#[extra.0>=10].first").String() == "Dale")
+
+	// strings
+	assert(t, Get(json, `i*.f*.#[extra.0<"11"].first`).Exists())
+	assert(t, Get(json, `i*.f*.#[first>"Dale"].last`).String() == "Craig")
+	assert(t, Get(json, `i*.f*.#[first>="Dale"].last`).String() == "Murphy")
+	assert(t, Get(json, `i*.f*.#[first="Dale"].last`).String() == "Murphy")
+	assert(t, Get(json, `i*.f*.#[first!="Dale"].last`).String() == "Craig")
+	assert(t, !Get(json, `i*.f*.#[first<"Dale"].last`).Exists())
+	assert(t, Get(json, `i*.f*.#[first<="Dale"].last`).Exists())
+	assert(t, Get(json, `i*.f*.#[first%"Da*"].last`).Exists())
+	assert(t, Get(json, `i*.f*.#[first%"Dale"].last`).Exists())
+	assert(t, Get(json, `i*.f*.#[first%"*a*"]#|#`).String() == "1")
+	assert(t, Get(json, `i*.f*.#[first%"*e*"]#|#`).String() == "2")
+	assert(t, Get(json, `i*.f*.#[first!%"*e*"]#|#`).String() == "0")
+
+	// trues
+	assert(t, Get(json, `i*.f*.#[cust1=true].first`).String() == "Dale")
+	assert(t, Get(json, `i*.f*.#[cust2=false].first`).String() == "Roger")
+	assert(t, Get(json, `i*.f*.#[cust1!=false].first`).String() == "Dale")
+	assert(t, Get(json, `i*.f*.#[cust2!=true].first`).String() == "Roger")
+	assert(t, !Get(json, `i*.f*.#[cust1>true].first`).Exists())
+	assert(t, Get(json, `i*.f*.#[cust1>=true].first`).Exists())
+	assert(t, !Get(json, `i*.f*.#[cust2<false].first`).Exists())
+	assert(t, Get(json, `i*.f*.#[cust2<=false].first`).Exists())
+
+}
+
+func TestQueryArrayValues(t *testing.T) {
+	json := `{
+		"artists": [
+			["Bob Dylan"],
+			"John Lennon",
+			"Mick Jagger",
+			"Elton John",
+			"Michael Jackson",
+			"John Smith",
+			true,
+			123,
+			456,
+			false,
+			null
+		]
+	}`
+	assert(t, Get(json, `a*.#[0="Bob Dylan"]#|#`).String() == "1")
+	assert(t, Get(json, `a*.#[0="Bob Dylan 2"]#|#`).String() == "0")
+	assert(t, Get(json, `a*.#[%"John*"]#|#`).String() == "2")
+	assert(t, Get(json, `a*.#[_%"John*"]#|#`).String() == "0")
+	assert(t, Get(json, `a*.#[="123"]#|#`).String() == "1")
+}
+
+func TestParenQueries(t *testing.T) {
+	json := `{
+		"friends": [{"a":10},{"a":20},{"a":30},{"a":40}]
+	}`
+	assert(t, Get(json, "friends.#(a>9)#|#").Int() == 4)
+	assert(t, Get(json, "friends.#(a>10)#|#").Int() == 3)
+	assert(t, Get(json, "friends.#(a>40)#|#").Int() == 0)
+}
+
+func TestSubSelectors(t *testing.T) {
+	json := `{
+		"info": {
+			"friends": [
+				{
+					"first": "Dale", "last": "Murphy", "kind": "Person",
+					"cust1": true,
+					"extra": [10,20,30],
+					"details": {
+						"city": "Tempe",
+						"state": "Arizona"
+					}
+				},
+				{
+					"first": "Roger", "last": "Craig", "kind": "Person",
+					"cust2": false,
+					"extra": [40,50,60],
+					"details": {
+						"city": "Phoenix",
+						"state": "Arizona"
+					}
+				}
+			]
+		}
+	  }`
+	assert(t, Get(json, "[]").String() == "[]")
+	assert(t, Get(json, "{}").String() == "{}")
+	res := Get(json, `{`+
+		`abc:info.friends.0.first,`+
+		`info.friends.1.last,`+
+		`"a`+"\r"+`a":info.friends.0.kind,`+
+		`"abc":info.friends.1.kind,`+
+		`{123:info.friends.1.cust2},`+
+		`[info.friends.#[details.city="Phoenix"]#|#]`+
+		`}.@pretty.@ugly`).String()
+	// println(res)
+	// {"abc":"Dale","last":"Craig","\"a\ra\"":"Person","_":{"123":false},"_":[1]}
+	assert(t, Get(res, "abc").String() == "Dale")
+	assert(t, Get(res, "last").String() == "Craig")
+	assert(t, Get(res, "\"a\ra\"").String() == "Person")
+	assert(t, Get(res, "@reverse.abc").String() == "Person")
+	assert(t, Get(res, "_.123").String() == "false")
+	assert(t, Get(res, "@reverse._.0").String() == "1")
+	assert(t, Get(json, "info.friends.[0.first,1.extra.0]").String() ==
+		`["Dale",40]`)
+	assert(t, Get(json, "info.friends.#.[first,extra.0]").String() ==
+		`[["Dale",10],["Roger",40]]`)
+}
+
+func TestArrayCountRawOutput(t *testing.T) {
+	assert(t, Get(`[1,2,3,4]`, "#").Raw == "4")
+}
+
+func TestParseQuery(t *testing.T) {
+	var path, op, value, remain string
+	var ok bool
+
+	path, op, value, remain, _, ok =
+		parseQuery(`#(service_roles.#(=="one").()==asdf).cap`)
+	assert(t, ok &&
+		path == `service_roles.#(=="one").()` &&
+		op == "=" &&
+		value == `asdf` &&
+		remain == `.cap`)
+
+	path, op, value, remain, _, ok = parseQuery(`#(first_name%"Murphy").last`)
+	assert(t, ok &&
+		path == `first_name` &&
+		op == `%` &&
+		value == `"Murphy"` &&
+		remain == `.last`)
+
+	path, op, value, remain, _, ok = parseQuery(`#( first_name !% "Murphy" ).last`)
+	assert(t, ok &&
+		path == `first_name` &&
+		op == `!%` &&
+		value == `"Murphy"` &&
+		remain == `.last`)
+
+	path, op, value, remain, _, ok = parseQuery(`#(service_roles.#(=="one"))`)
+	assert(t, ok &&
+		path == `service_roles.#(=="one")` &&
+		op == `` &&
+		value == `` &&
+		remain == ``)
+
+	path, op, value, remain, _, ok =
+		parseQuery(`#(a\("\"(".#(=="o\"(ne")%"ab\")").remain`)
+	assert(t, ok &&
+		path == `a\("\"(".#(=="o\"(ne")` &&
+		op == "%" &&
+		value == `"ab\")"` &&
+		remain == `.remain`)
+}
+
+func TestParentSubQuery(t *testing.T) {
+	var json = `{
+		"topology": {
+		  "instances": [
+			{
+			  "service_version": "1.2.3",
+			  "service_locale": {"lang": "en"},
+			  "service_roles": ["one", "two"]
+			},
+			{
+			  "service_version": "1.2.4",
+			  "service_locale": {"lang": "th"},
+			  "service_roles": ["three", "four"]
+			},
+			{
+			  "service_version": "1.2.2",
+			  "service_locale": {"lang": "en"},
+			  "service_roles": ["one"]
+			}
+		  ]
+		}
+	  }`
+	res := Get(json, `topology.instances.#( service_roles.#(=="one"))#.service_version`)
+	// should return two instances
+	assert(t, res.String() == `["1.2.3","1.2.2"]`)
+}
+
+func TestSingleModifier(t *testing.T) {
+	var data = `{"@key": "value"}`
+	assert(t, Get(data, "@key").String() == "value")
+	assert(t, Get(data, "\\@key").String() == "value")
+}
+
+func TestModifiersInMultipaths(t *testing.T) {
+	AddModifier("case", func(json, arg string) string {
+		if arg == "upper" {
+			return strings.ToUpper(json)
+		}
+		if arg == "lower" {
+			return strings.ToLower(json)
+		}
+		return json
+	})
+	json := `{"friends": [
+		{"age": 44, "first": "Dale", "last": "Murphy"},
+		{"age": 68, "first": "Roger", "last": "Craig"},
+		{"age": 47, "first": "Jane", "last": "Murphy"}
+	]}`
+
+	res := Get(json, `friends.#.{age,first|@case:upper}|@ugly`)
+	exp := `[{"age":44,"@case:upper":"DALE"},{"age":68,"@case:upper":"ROGER"},{"age":47,"@case:upper":"JANE"}]`
+	assert(t, res.Raw == exp)
+
+	res = Get(json, `{friends.#.{age,first:first|@case:upper}|0.first}`)
+	exp = `{"first":"DALE"}`
+	assert(t, res.Raw == exp)
+
+}
+
+func TestIssue141(t *testing.T) {
+	json := `{"data": [{"q": 11, "w": 12}, {"q": 21, "w": 22}, {"q": 31, "w": 32} ], "sql": "some stuff here"}`
+	assert(t, Get(json, "data.#").Int() == 3)
+	assert(t, Get(json, "data.#.{q}|@ugly").Raw == `[{"q":11},{"q":21},{"q":31}]`)
+	assert(t, Get(json, "data.#.q|@ugly").Raw == `[11,21,31]`)
+}
+
+func TestChainedModifierStringArgs(t *testing.T) {
+	// issue #143
+	AddModifier("push", func(json, arg string) string {
+		json = strings.TrimSpace(json)
+		if len(json) < 2 || !Parse(json).IsArray() {
+			return json
+		}
+		json = strings.TrimSpace(json[1 : len(json)-1])
+		if len(json) == 0 {
+			return "[" + arg + "]"
+		}
+		return "[" + json + "," + arg + "]"
+	})
+	res := Get("[]", `@push:"2"|@push:"3"|@push:{"a":"b","c":["e","f"]}|@push:true|@push:10.23`)
+	assert(t, res.String() == `["2","3",{"a":"b","c":["e","f"]},true,10.23]`)
+}
+
+func TestFlatten(t *testing.T) {
+	json := `[1,[2],[3,4],[5,[6,[7]]],{"hi":"there"},8,[9]]`
+	assert(t, Get(json, "@flatten").String() == `[1,2,3,4,5,[6,[7]],{"hi":"there"},8,9]`)
+	assert(t, Get(json, `@flatten:{"deep":true}`).String() == `[1,2,3,4,5,6,7,{"hi":"there"},8,9]`)
+	assert(t, Get(`{"9999":1234}`, "@flatten").String() == `{"9999":1234}`)
+}
+
+func TestJoin(t *testing.T) {
+	assert(t, Get(`[{},{}]`, "@join").String() == `{}`)
+	assert(t, Get(`[{"a":1},{"b":2}]`, "@join").String() == `{"a":1,"b":2}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2}]`, "@join").String() == `{"a":1,"b":2}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2},5,{"c":3}]`, "@join").String() == `{"a":1,"b":2,"c":3}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2},5,{"c":3}]`, `@join:{"preserve":true}`).String() == `{"a":1,"b":1,"b":2,"c":3}`)
+	assert(t, Get(`[{"a":1,"b":1},{"b":2},5,{"c":3}]`, `@join:{"preserve":true}.b`).String() == `1`)
+	assert(t, Get(`{"9999":1234}`, "@join").String() == `{"9999":1234}`)
+}
+
+func TestValid(t *testing.T) {
+	assert(t, Get("[{}", "@valid").Exists() == false)
+	assert(t, Get("[{}]", "@valid").Exists() == true)
+}
+
+// https://github.com/tidwall/gjson/issues/152
+func TestJoin152(t *testing.T) {
+	var json = `{
+		"distance": 1374.0,
+		"validFrom": "2005-11-14",
+		"historical": {
+		  "type": "Day",
+		  "name": "last25Hours",
+		  "summary": {
+			"units": {
+			  "temperature": "C",
+			  "wind": "m/s",
+			  "snow": "cm",
+			  "precipitation": "mm"
+			},
+			"days": [
+			  {
+				"time": "2020-02-08",
+				"hours": [
+				  {
+					"temperature": {
+					  "min": -2.0,
+					  "max": -1.6,
+					  "value": -1.6
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 92.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-08T16:00:00+01:00"
+				  },
+				  {
+					"temperature": {
+					  "min": -1.7,
+					  "max": -1.3,
+					  "value": -1.3
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 92.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-08T17:00:00+01:00"
+				  },
+				  {
+					"temperature": {
+					  "min": -1.3,
+					  "max": -0.9,
+					  "value": -1.2
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 91.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-08T18:00:00+01:00"
+				  }
+				]
+			  },
+			  {
+				"time": "2020-02-09",
+				"hours": [
+				  {
+					"temperature": {
+					  "min": -1.7,
+					  "max": -0.9,
+					  "value": -1.5
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 91.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-09T00:00:00+01:00"
+				  },
+				  {
+					"temperature": {
+					  "min": -1.5,
+					  "max": 0.9,
+					  "value": 0.2
+					},
+					"wind": {},
+					"precipitation": {},
+					"humidity": {
+					  "value": 67.0
+					},
+					"snow": {
+					  "depth": 49.0
+					},
+					"time": "2020-02-09T01:00:00+01:00"
+				  }
+				]
+			  }
+			]
+		  }
+		}
+	  }`
+
+	res := Get(json, "historical.summary.days.#.hours|@flatten|#.humidity.value")
+	assert(t, res.Raw == `[92.0,92.0,91.0,91.0,67.0]`)
+}
+
+func TestVariousFuzz(t *testing.T) {
+	// Issue #192	assert(t, squash(`"000"hello`) == `"000"`)
+	assert(t, squash(`"000"`) == `"000"`)
+	assert(t, squash(`"000`) == `"000`)
+	assert(t, squash(`"`) == `"`)
+
+	assert(t, squash(`[000]hello`) == `[000]`)
+	assert(t, squash(`[000]`) == `[000]`)
+	assert(t, squash(`[000`) == `[000`)
+	assert(t, squash(`[`) == `[`)
+	assert(t, squash(`]`) == `]`)
+
+	testJSON := `0.#[[{}]].@valid:"000`
+	Get(testJSON, testJSON)
+
+	// Issue #195
+	testJSON = `\************************************` +
+		`**********{**",**,,**,**,**,**,"",**,**,**,**,**,**,**,**,**,**]`
+	Get(testJSON, testJSON)
+
+	// Issue #196
+	testJSON = `[#.@pretty.@join:{""[]""preserve"3,"][{]]]`
+	Get(testJSON, testJSON)
+
+}
